<?php
/*
Plugin Name: Fast Static Feed
Plugin URI: http://www.g-loaded.eu/2008/05/10/fast-static-feed-wordpress-plugin/
Description: Caches the entries feed and delivers the static XML data instead of re-generating it on every client request.
Version: 0.2.0
Author: George Notaras
Author URI: http://www.g-loaded.eu/
*/

/**
 *  Copyright 2008 George Notaras <gnot [at] g-loaded.eu>, CodeTRAX.org 
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License. 
 */

//
// Configuration Begin
//

// The default BASE DIRECTORY is:  wp-content/

// Beware!!
// *********************************************************************
// Make sure the directory above is WRITABLE by the web server.
// This is the place where the cache directory (cache-xml) will be
// created. If you fail to do so, the plugin will not work as expected
// and, whenever a client asks for your feed, it will receive
// a '404 Not Found' HTTP error code. - You have been warned!
// ********************************************************************

// By default, the WordPress feeds contain elements of the 'wfw' namespace
// (CommentAPI), so a feed reader can fetch each feed entry's comments.
// By setting the following to 1, these elements will be stripped
// from the static feed.
// It is recommended to set this to 1 in order to save bandwidth.
// More info: http://wellformedweb.org/news/wfw_namespace_elements/
$FSF_STRIP_COMMENTS = 0;

// Set the following to 1 and various messages will be recorded in the logfile,
// which is located in the cache dir (wp-content/cache-xml/)
$FSF_DEBUG = 0;

//
// Configuration End
//



// Internal Constants - (!!) Edit at your own risk!

// Extra argument to force feed generation
$FSF_FORCE_GEN_ARG = "force_feed_generation";

// Feed locations
$FSF_FEED_URIs = array( "/feed/", "/feed/atom/", "/feed/rdf/", "/feed/rss/");

// Cache directory
$FSF_CACHE_DIR = ABSPATH . "wp-content/" . "cache-xml/";


/**
 * Takes the feed REQUEST URI and transforms it to a /path/to/feedcache.xml
 * Procedure:
 * 1. Strip leading/trailing slash
 * 2. Replace internal slashes to dots
 * 3. Add the ".xml" extension
 * 4. Prefix with the default output directory
 * 
 * 'feed' is replaced by 'rss2' in case of the RSS 2.0 feed.
 */
function fsf_feed_uri2path($req_uri) {
	global $FSF_CACHE_DIR;
	$req_uri = trim($req_uri, "/");
	$req_uri = str_replace("/", ".", $req_uri);
	if ($req_uri == "feed") {
		$req_uri .= ".rss2";
	}
	$req_uri .= ".xml";
	return $FSF_CACHE_DIR . $req_uri;
}


/**
 * Accepts a feed type (feed, atom, rdf, rss) and returns
 * the static file path
 */
function fsf_feed_name2path($feed_name) {
	global $FSF_CACHE_DIR;
	if ($feed_name == "feed") {
		$feed_name = "rss2";
	}
	return $FSF_CACHE_DIR . "feed." . $feed_name . ".xml";
}


/**
 * Reads the feed dump and returns the data
 * @feed_dump_path : The path to the file that contains the dumped feed data.
 * @return string The contents of the feed dump.
 */
function fsf_get_xml_content($feed_dump_path) {
	$handle = fopen($feed_dump_path, "r");
	$xml_data = fread($handle, filesize($feed_dump_path));
	fclose($handle);
	return $xml_data;
}


/**
 * Sends the response to the client
 * 
 * @feed_dump_path : The path to the static file.
 * @return nothing
 * 
 * Possible responses:
 * 404 : Whenever it cannot find the static file.
 *       It performs 2 checks:
 *       1 - If the first check fails to find the static file,
 *           fsf_auto_create_static_feed() is called to try to generate it.
 *       2 - If the second check does not find the static file too, it
 *           is assumed that there was a problem and a '404 Not Found' error
 *           response is returned.
 * 304 : if the client provides the "If-Modified-Since" HTTP header, it checks
 *       the dump's last modification time and if they match, then a
 *       'Not Modified' response is returned.
 * 200 : The contents of the dump are returned normally.
 */
function fsf_send_response($feed_dump_path) {
	// 1st check for static file existence
	if ( !file_exists($feed_dump_path) ) {
		fsf_auto_create_static_feed();
		sleep(1);
		// 2nd check
		if ( !file_exists($feed_dump_path) ) {
			header("HTTP/1.0 404 Not Found");
			exit();
		}
	}
	$lastModServer = filemtime($feed_dump_path);
	$LastModRequest = "";
	if ( isset($_SERVER["HTTP_IF_MODIFIED_SINCE"]) ) {
		$LastModRequest = strtotime($_SERVER["HTTP_IF_MODIFIED_SINCE"]);
	}
	if ( $LastModRequest && ($LastModRequest == $lastModServer) ) {
		header("Last-Modified: " . gmdate("D, d M Y H:i:s", $lastModServer) . " GMT", true, 304);
		exit();
	} else {
		$xml_data = fsf_get_xml_content($feed_dump_path);
		header("Last-Modified: " . gmdate("D, d M Y H:i:s", $lastModServer) . " GMT", true, 200);
		header("Content-Length: " . strlen($xml_data));
		header("Content-Type: text/xml; charset=" . get_bloginfo("charset"));
		echo $xml_data;
		exit();
	}
}


/**
 * Checks whether the URI is one of the rss2|atom|rdf|rss feeds
 * and calls fsf_send_response() to return the static content to
 * the client.
 */
function fsf_redirect_feeds() {
	global $feed, $FSF_FORCE_GEN_ARG;
	global $wp, $withcomments;

	if (	is_feed()
		&& $feed != 'comments-rss2'
		&& !is_single()
		&& !$wp->query_vars['category_name']
		&& !$wp->query_vars['tag']
		&& !$wp->query_vars['author_name']
		&& !$wp->query_vars['year']
		&& ($withcomments != 1)
		&& !isset($_GET[$FSF_FORCE_GEN_ARG])
		) {
//		fsf_log(print_r($_SERVER, true));
//		fsf_log(print_r($_GET, true));
		fsf_log("Delivering Static Feed: " . $_SERVER["REQUEST_URI"]);
		$feed_dump_path = fsf_feed_name2path($feed);
		//fsf_log($feed_dump_path);
		fsf_send_response($feed_dump_path);
	}
	if ( is_feed() ) { fsf_log("Dynamically generated: " . $_SERVER["REQUEST_URI"]); }
}

// Redirects feeds to serve the static content
add_action('template_redirect', 'fsf_redirect_feeds');



/**
 * Automatic Feed Dumping
 *
 * The RSS 2.0 feed gets dumped to the specified file
 * every time:
 *  - a post is published
 *  - an already published post is edited
 *  - a post is deleted
 */
function fsf_auto_create_static_feed() {
	global $FSF_STRIP_COMMENTS, $FSF_FORCE_GEN_ARG;
	global $FSF_FEED_URIs, $FSF_CACHE_DIR;

	fsf_check_cache_dir();

	foreach ($FSF_FEED_URIs as $feed_uri) {
		// An extra argument is added (force_feed_generation=yes)
		// This is used in order to avoid looping on the cached XML data
		// when fresh feed content is needed to be fetched.
		$extra_arg = "?".$FSF_FORCE_GEN_ARG."=yes";
		// Read the feed
		$rss_web = fopen(get_option("siteurl") . $feed_uri . $extra_arg, "r");
		$xml_dump = "";
		if ($rss_web) {
			while ( !feof($rss_web) ) {
				$line = fgets($rss_web, 4096);
				// Strips the extra argument in case it is used in any url that is generated by WP
				$line = str_replace($extra_arg, "", $line);
				if (!$FSF_STRIP_COMMENTS or !strpos($line, "wfw:commentRss")) {
					$xml_dump .= $line;
				}
			}
		}
		fclose($rss_web);
		// Dump to file
		$rss_dump = fopen(fsf_feed_uri2path($feed_uri), "w");
		fwrite($rss_dump, $xml_dump);
		fclose($rss_dump);
	}
}


/**
 * Checks if the cache dir exists and, if not, it creates it.
 * Also creates a dummy index.php file inside the cache dir, so to prevent directory listings.
 */
function fsf_check_cache_dir() {
// Check if cache dir exists. If not, create it.
	global $FSF_CACHE_DIR;
	if ( !file_exists($FSF_CACHE_DIR) ) {
		// Save the old umask and set the current to 0, so to create the dir with 0777
		$old_umask = umask(0);
		mkdir($FSF_CACHE_DIR, 0777);
		// Create an empty index.php
		$handle = fopen($FSF_CACHE_DIR . "index.php", "w");
		fwrite($handle, "<?php /* Index */ ?>\n");
		fclose($handle);
		// Revert the umask
		umask($old_umask);
	}
}


/**
 * The Fast-Static-Feed logger
 * Requires to set the $FSF_DEBUG option to 1 to record messages.
 */
function fsf_log($msg) {
	global $FSF_DEBUG, $FSF_CACHE_DIR;
	if ($FSF_DEBUG) {
		$filename = $FSF_CACHE_DIR . "fast-static-feed.log";
		$handle = fopen($filename, "a");
		fwrite($handle, "$msg\n");
		fclose($handle);
	}
}


add_action('delete_post', "fsf_auto_create_static_feed", 1000);
add_action('publish_post', "fsf_auto_create_static_feed", 1000);
// The following is not required: All edits are processed  by the 'publish_post' hook.
//add_action('edit_post', "fsf_auto_create_static_feed", 9999);

?>
